#!/usr/bin/perl -T -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults

use Cwd qw (cwd); # for cwd(),



# The zeroth argument will be replaced in daemonize().
my $ORIG_ARG0;# = $0;
# Getopt::Long clears all arguments it processed (untaint both @ARGVs here!)
my @ORIG_ARGV;# = @ARGV;
# daemonize() switches to the root later on and we need to come back here
# somehow -- untaint the dir to be on the safe side.
my $ORIG_CWD;# = Cwd::cwd();



##########################################################################
# sub daemon_init($0, @ARGV)
# global   -----
# @param   string $0
# @param   LIST   @ARGV
# @return  void
sub daemon_init {
	$ORIG_ARG0=shift(@_);
	@ORIG_ARGV=@_;
	$ORIG_CWD=Cwd::cwd();
}



##########################################################################
# sub daemon_setps($pscmd)
# global   -----
# @param   string $pscmd
# @return  void
sub daemon_setps {
	my ($pscmd)=@_;
	
	if (!defined($pscmd)) { $pscmd=join(' ', $ORIG_ARG0, @ORIG_ARGV); }
	# Pretty command line in ps
	$0 = $pscmd;
}



##########################################################################
# sub daemon_run(\&callback, $pidfile [, arg1[, arg2[, arg3[, ...]]]])
# global   -----
# @param   ref    \&callback
# @param   ... args passed into callback
# @return  void
sub daemon_run {
	my ($callback)=shift(@_);
	my ($pidfile)=shift(@_);
	my ($retstatus, $pid);
	
	$pid = fork();
	if ($pid) {
		# PARENT
		#print("PARENT\n");
		if (open(FILE,">$pidfile")) { print(FILE $pid); close(FILE); }
		return 0;
	} else {
		# Redirect in and out to the bit bucket
		open(STDIN,  "</dev/null") or die("$ORIG_ARG0: cannot read from /dev/null: $!\n");
		open(STDOUT, ">/dev/null") or die("$ORIG_ARG0: cannot write to /dev/null: $!\n");
		# call back
		$pid=&{$callback}(@_);
		if (!defined($pid)) { $pid=0; }
		exit($pid);
	};
}



1; # return status for include mode


