#!/usr/bin/perl -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults

use File::Basename; # for basename(),
use POSIX qw(strftime); # for strftime(),
use Fcntl qw(:DEFAULT :flock); # import LOCK_* constants, O_NONBLOCK
use Sys::Hostname; # for hostname(),

use Socket; # for gethostbyname(), gethostbyaddr(), getprotobyname(), getprotobynumber(), getservbyname(), getprotobynumber(), sockaddr_in(), getpeername(), inet_ntoa(), inet_aton(),
use IO::Socket; # for IO::Socket::INET->new(),



# code from:
# RPM: perl-SMTP-Server-1.1-1.el5.pp.noarch.rpm
# package Net::SMTP::Server::Client;



my (%PCFCONF);

$PCFCONF{'HOSTNAME'}=$ENV{'HOSTNAME'};
if (!defined($PCFCONF{'HOSTNAME'})||length($PCFCONF{'HOSTNAME'})==0) { $PCFCONF{'HOSTNAME'}=hostname(); }
$PCFCONF{'Banner'}="$PCFCONF{'HOSTNAME'} SMTP Perl-Content-Filter";
$PCFCONF{'Timeout'}=30;



$PCFCONF{'verbose'}=0;
$PCFCONF{'ErrorCMDS'}='502 5.5.2 Error: command not recognized';
$PCFCONF{'ErrorNeedHELO'}='503 5.5.1 Error: send HELO/EHLO first';
$PCFCONF{'ErrorNeedMAIL'}='503 5.5.1 Error: need MAIL command';
$PCFCONF{'ErrorNeedRCPT'}='503 5.5.1 Error: need RCPT command';
$PCFCONF{'ErrorSyntaxCMDS'}='500 5.5.2 Error: bad syntax';
$PCFCONF{'ErrorSyntaxHELO'}='501 Syntax: HELO hostname';
$PCFCONF{'ErrorSyntaxMAIL'}='501 5.5.4 Syntax: MAIL FROM:<address>';
$PCFCONF{'ErrorSyntaxRCPT'}='501 5.5.4 Syntax: RCPT TO:<address>';
$PCFCONF{'TitleARGS'}=0;
$PCFCONF{'TitleCONN'}=undef;
$PCFCONF{'TitleHELP'}=undef;
$PCFCONF{'TitleHELO'}=undef;
$PCFCONF{'TitleMAIL'}=undef;
$PCFCONF{'TitleRCPT'}=undef;
$PCFCONF{'TitleDATA'}=undef;
$PCFCONF{'TitleENDOF'}=undef;



# cmd callbacks (must be defined before server run, because defined after causes not defined SMTP CMDS)
my %SMTPD_CMDS = (
	'DATA' => \&smtpd_cmd_data,
	'EXPN' => \&smtpd_cmd_noway,
	'HELO' => \&smtpd_cmd_helo,
	'EHLO' => \&smtpd_cmd_helo,
	'HELP' => \&smtpd_cmd_help,
	'MAIL' => \&smtpd_cmd_mail,
	'NOOP' => \&smtpd_cmd_noop,
	'QUIT' => \&smtpd_cmd_quit,
	'RCPT' => \&smtpd_cmd_rcpt,
	'RSET' => \&smtpd_cmd_reset,
	'VRFY' => \&smtpd_cmd_noway
);



# New instance.
sub smtpd_new {
	my ($sock) = shift(@_);
	my ($REFSMTPD);
	my ($key, %ARGS);
	
	%{$REFSMTPD} = ();
	$REFSMTPD->{STATE} = 'CONN'; # added
	$REFSMTPD->{HELO} = undef; # added
	$REFSMTPD->{FROM} = undef;
	@{ $REFSMTPD->{TO} } = ();
	$REFSMTPD->{DATA} = undef; # changed MSG to DATA
	
	%ARGS=@_;
	for $key (keys(%ARGS)) {
		#print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): |$key|=|$ARGS{$key}|\n");
		if ($key eq 'Banner' ) { $PCFCONF{'Banner'} =$ARGS{$key};      }
		if ($key eq 'Timeout') { $PCFCONF{'Timeout'}=int($ARGS{$key}); }
		
		if ($key eq 'ErrorNeedHELO') { $PCFCONF{'ErrorNeedHELO'}=$ARGS{$key}; }
		if ($key eq 'ErrorNeedMAIL') { $PCFCONF{'ErrorNeedMAIL'}=$ARGS{$key}; }
		if ($key eq 'ErrorNeedRCPT') { $PCFCONF{'ErrorNeedRCPT'}=$ARGS{$key}; }
		
		if ($key eq 'TitleARGS') { $PCFCONF{'TitleARGS'}=1; }
		if ($key eq 'TitleCONN') { $PCFCONF{'TitleCONN'}=$ARGS{$key}; }
		if ($key eq 'TitleHELP') { $PCFCONF{'TitleHELP'}=$ARGS{$key}; }
		if ($key eq 'TitleHELO') { $PCFCONF{'TitleHELO'}=$ARGS{$key}; }
		if ($key eq 'TitleMAIL') { $PCFCONF{'TitleMAIL'}=$ARGS{$key}; }
		if ($key eq 'TitleRCPT') { $PCFCONF{'TitleRCPT'}=$ARGS{$key}; }
		if ($key eq 'TitleDATA') { $PCFCONF{'TitleDATA'}=$ARGS{$key}; }
	}
	
	$sock->timeout($PCFCONF{'Timeout'});
	
	###if (!defined($SMTPD_CMDS{'HELO'})) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): NOT DEFINED cmd=|HELO|\n"); }
	# in main loop this is catched
	#unless (defined($sock)) { print(STDERR "No client connection specified."); return undef; }
	#if (!defined($sock)) { print(STDERR "No client connection specified."); return undef; }
	if (defined($PCFCONF{'TitleCONN'})) { $0=$PCFCONF{'TitleCONN'}; }
	smtpd_put($sock, '220 '.$PCFCONF{'Banner'});
	return $REFSMTPD;
}



# Utility functions.
sub smtpd_sockrecv {
	my ($sock, $REFSMTPD, $queueid) = @_;
	# @_ = ARGS call function
	my ($line, $cmdstr, @args, $result, $sp1);
	
	$REFSMTPD->{QUEUEID}=$queueid;
	while ($line=<$sock>) {
		# Clean up.
		chomp($line);
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		
		# goto bad unless length($_);
		if (length($line)==0) {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): if (!length(line)==0)\n"); }
			smtpd_put($sock, $PCFCONF{'ErrorSyntaxCMDS'});
			next;
		}
		
		$sp1=index($line, ':');
		if ($sp1>0 && substr($line,$sp1+1,1) ne ' ') { $line=substr($line,0,$sp1+1).' '.substr($line,$sp1+1); }
		#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): line=|$line|\n"); }
		@args=split(/\s+/, $line);
		($cmdstr)=shift(@args);
		$cmdstr =~ tr/a-z/A-Z/;
		#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): cmdstr=|$cmdstr| @args=|@args|\n"); }
		
		if (!defined($SMTPD_CMDS{$cmdstr})) {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): NOT DEFINED: CMDS{$cmdstr} !!!\n"); }
			#bad:
			smtpd_put($sock, $PCFCONF{'ErrorCMDS'});
			next;
		}
		
		#return (defined($REFSMTPD->{MSG}) ? 1 : 0) unless &{$SMTPD_CMDS{$cmdstr}}($sock, $REFSMTPD, \@args);
		#unless &{$SMTPD_CMDS{$cmdstr}}($sock, $REFSMTPD, \@args) { return (defined($REFSMTPD->{MSG}) ? 1 : 0); }
		#if (!&{$SMTPD_CMDS{$cmdstr}}($sock,$REFSMTPD,\@args)) { return (defined($REFSMTPD->{MSG}) ? 1 : 0); }
		#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): {\$SMTPD_CMDS{$cmdstr}}($sock,$REFSMTPD,|@args|):\n"); }
		$result=&{$SMTPD_CMDS{$cmdstr}}($sock,$REFSMTPD,\@args);
		if ($result<0) {
			###$sock->close(); # 2013/11/08: comment out: to enable: before-queue content filtering
			delete($REFSMTPD->{QUEUEID});
			return -1;
		}
		if ($result==0) {
			###$sock->close(); # 2013/11/08: comment out: to enable: before-queue content filtering
			delete($REFSMTPD->{QUEUEID});
			if (defined($REFSMTPD->{DATA})) { return 1; } # changed MSG to DATA
			else { return 0; }
		}
	}
	###$sock->close(); # 2013/11/08: comment out: to enable: before-queue content filtering
	delete($REFSMTPD->{QUEUEID});
	return -1;
}

sub smtpd_from_toto {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	my ($which, $var, $args) = @_;
	
	if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): which=$which, args=|".join('|',@{$args})."|\n"); }
	if (!(${$args}[0] =~ /^$which\s*([^\s]+)/i)) {
		#if (!${$args}[1] || !(${$args}[0] =~ /^$which$/i))
		if (!(${$args}[0] =~ /^$which$/i) || !defined(${$args}[1])) {
			smtpd_put($sock, $PCFCONF{'ErrorCMDS'});
			return -1;
		}
		
		if (ref($var) eq 'ARRAY') {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): which=|$which| ARRAY: args[0]=|".${$args}[0]."| args[1]=|".${$args}[1]."|\n"); }
			push(@{$var}, ${$args}[1]);
		} else {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): which=|$which| ELSE:  args[0]=|".${$args}[0]."| args[1]=|".${$args}[1]."|\n"); }
			${$var}=${$args}[1];
		}
	}
	
	#unless (!defined($1))
	if (defined($1)) {
		if (ref($var) eq 'ARRAY') {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): which=|$which| ARRAY: args=$1\n"); }
			push(@{$var}, $1);
		} else {
			#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): which=|$which| ELSE:  args=$1\n"); }
			${$var}=$1;
		}
	}
	
	if    ($which eq 'MAIL') { smtpd_put($sock, "250 2.1.0 Ok"); }
	elsif ($which eq 'RCPT') { smtpd_put($sock, "250 2.1.5 Ok"); }
	else                     { smtpd_put($sock, "250 2.2.0 Ok"); }
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_put {
	my ($sock)=shift(@_);
	# @_ = ARGS call function
	
	if ($sock->connected()) { $sock->send(join("\r\n",@_)); }
	if ($sock->connected()) { $sock->send("\r\n"); }
}



# cmd functions.
sub smtpd_cmd_data_old {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	my $done = undef;
	my ($line, $lnpr);
	
	if (!defined($REFSMTPD->{FROM})) { smtpd_put($sock, "503 5.5.1 Error: need MAIL command"); return 1; }
	if (!defined(@{$REFSMTPD->{TO}})) { smtpd_put($sock, "503 5.5.1 Error: need RCPT command");  return 1; }
	
	smtpd_put($sock, "354 End data with <CR><LF>.<CR><LF>");
	
	while ($line=<$sock>) {
		$lnpr=$line;
		$lnpr=~s,[\t\r\n]*$,,g;
		#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): line=|$lnpr|\n"); }
		if ($line=~m/^\.\r\n$/) { $done = 1; last; }
		if ($line=~m/^\.\n$/  ) { $done = 1; last; }
		
		# RFC 821 compliance.
		$line=~s/^\.\./\./;
		$REFSMTPD->{DATA} .= $line; # changed MSG to DATA
	}
	
	if (!defined($done)) {
		smtpd_put($sock, "502 5.5.2 Error: command not recognized");
		return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
	}
	
	###smtpd_put($sock, "250 Ok: queued as ".$REFSMTPD->{QUEUEID});
}

sub smtpd_cmd_data {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	my ($data, $state);
	
	#if (!defined($REFSMTPD->{HELO}) || length($REFSMTPD->{HELO})<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedHELO'}); return 1; }
	#if (!defined($REFSMTPD->{FROM}) || length($REFSMTPD->{FROM})<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedMAIL'}); return 1; }
	if (!defined(@{$REFSMTPD->{TO}}) || length(join('',@{$REFSMTPD->{TO}}))<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedRCPT'}); return 1; }
	
	if (defined($PCFCONF{'TitleDATA'})) { $0=$PCFCONF{'TitleDATA'}; }
	$REFSMTPD->{STATE}='DATA';
	smtpd_put($sock, "354 End data with <CR><LF>.<CR><LF>");
	$state=0;
	$REFSMTPD->{DATA}='';
	while (1) {
		$sock->recv($data,52428800+1+1+1); # datasize (50MB) + newline + single_dot + newline
		if (!defined($data) || length($data)==0) { last; }
		$REFSMTPD->{DATA}=$REFSMTPD->{DATA}.$data;
		#if ($PCFCONF{'verbose'}==1) { print("\nDATA=\n|".$REFSMTPD->{DATA}."|\n\n"); }
		if      (substr($REFSMTPD->{DATA},length($REFSMTPD->{DATA})-3,3) eq "\x{0A}\x{2E}\x{0A}") {
			#my $end=substr($REFSMTPD->{DATA},length($REFSMTPD->{DATA})-3,3);
			#if ($PCFCONF{'verbose'}==1) { printf(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): END=%02X %02X %02X\n", ord(substr($end,0,1)), ord(substr($end,1,1)), ord(substr($end,2,1))); }
			$REFSMTPD->{DATA}=substr($REFSMTPD->{DATA}, 0, -2);
			$state=1;
			last;
		} elsif (substr($REFSMTPD->{DATA},length($REFSMTPD->{DATA})-5,5) eq "\x{0D}\x{0A}\x{2E}\x{0D}\x{0A}") {
			#my $end=substr($REFSMTPD->{DATA},length($REFSMTPD->{DATA})-5,5);
			#if ($PCFCONF{'verbose'}==1) { printf(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): END=%02X %02X %02X %02X %02X\n", ord(substr($end,0,1)), ord(substr($end,1,1)), ord(substr($end,2,1)), ord(substr($end,3,1)), ord(substr($end,4,1))); }
			$REFSMTPD->{DATA}=substr($REFSMTPD->{DATA}, 0, -3);
			$state=1;
			last;
		}
	}
	
	if ($state==1) {
		$REFSMTPD->{STATE}='END-OF-DATA';
		#smtpd_put($sock, "250 Ok: queued as ".$REFSMTPD->{QUEUEID});
		return 0; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
	} else {
		smtpd_put($sock, $PCFCONF{'ErrorCMDS'});
		return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
	}
}

sub smtpd_cmd_noway {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	$REFSMTPD->{STATE}='NOWAY';
	smtpd_put($sock, $PCFCONF{'ErrorCMDS'});
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_cmd_helo {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	if (defined($PCFCONF{'TitleHELO'})) { $0=$PCFCONF{'TitleHELO'}; }
	$REFSMTPD->{STATE}='HELO';
	if (!defined(${$_[0]}[0])) { smtpd_put($sock, $PCFCONF{'ErrorSyntaxHELO'}); return 1; }
	if (defined($PCFCONF{'TitleHELO'}) && $PCFCONF{'TitleARGS'}==1) { $0=$PCFCONF{'TitleHELO'}.' '.${$_[0]}[0]; }
	if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): ".join(' ',@{$_[0]})."\n"); }
	$REFSMTPD->{HELO}=${$_[0]}[0]; # added
	smtpd_put($sock, "250 $PCFCONF{'HOSTNAME'}");
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_cmd_help {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	my $i = 0;
	my $cmdstr = "214-Commands";
	my $total = keys(%SMTPD_CMDS);
	my $key;
	
	if (defined($PCFCONF{'TitleHELP'})) { $0=$PCFCONF{'TitleHELP'}; }
	$REFSMTPD->{STATE}='HELP';
	foreach $key (sort(keys(%SMTPD_CMDS))) {
		if (!($i++ % 5)) {
			if (($total - $i) < 5) {
				$cmdstr .= "\r\n214 ";
			} else {
				$cmdstr .= "\r\n214-";
			}
		} else {
			$cmdstr .= ' ';
		}
		
		$cmdstr .= $key;
	}
	
	smtpd_put($sock, $cmdstr);
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_cmd_mail {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	if (!defined($REFSMTPD->{HELO}) || length($REFSMTPD->{HELO})<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedHELO'}); return 1; }
	$REFSMTPD->{STATE}='MAIL';
	if (!defined(${$_[0]}[0]) || ${$_[0]}[0] ne 'FROM:' || !defined(${$_[0]}[1])) { smtpd_put($sock, $PCFCONF{'ErrorSyntaxMAIL'}); return 1; }
	if (defined($PCFCONF{'TitleMAIL'})) { $0=$PCFCONF{'TitleMAIL'}; }
	#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): ".join(' ',@{$_[0]})."\n"); }
	if (defined($PCFCONF{'TitleMAIL'}) && $PCFCONF{'TitleARGS'}==1) { my $line=join(' ',@{$_[0]}); if(substr($line,0,5) eq 'FROM:'){$line=substr($line,5);} $line=~s,^[\s\t]*,,g; $0=$PCFCONF{'TitleMAIL'}.' '.$line; }
	return smtpd_from_toto($sock, $REFSMTPD, 'FROM:', \$REFSMTPD->{FROM}, @_);
}

sub smtpd_cmd_noop {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	$REFSMTPD->{STATE}='NOOP';
	smtpd_put($sock, "250 Whatever.");
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_cmd_quit {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	$REFSMTPD->{STATE}='HELP';
	smtpd_put($sock, "221 Bye.");
	###$sock->close(); # 2013/11/08: comment out: to enable: before-queue content filtering
	return 0; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}

sub smtpd_cmd_rcpt {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	#if (!defined($REFSMTPD->{HELO}) || length($REFSMTPD->{HELO})<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedHELO'}); return 1; }
	if (!defined($REFSMTPD->{FROM}) || length($REFSMTPD->{FROM})<=0) { smtpd_put($sock, $PCFCONF{'ErrorNeedMAIL'}); return 1; }
	$REFSMTPD->{STATE}='RCPT';
	if (!defined(${$_[0]}[0]) || ${$_[0]}[0] ne 'TO:' || !defined(${$_[0]}[1])) { smtpd_put($sock, $PCFCONF{'ErrorSyntaxRCPT'}); return 1; }
	if (defined($PCFCONF{'TitleRCPT'})) { $0=$PCFCONF{'TitleRCPT'}; }
	#if ($PCFCONF{'verbose'}==1) { print(basename(__FILE__).":".__LINE__.":".(caller(0))[3]."(): ".join(' ',@{$_[0]})."\n"); }
	if (defined($PCFCONF{'TitleRCPT'}) && $PCFCONF{'TitleARGS'}==1) { my $line=join(' ',@{$_[0]}); if(substr($line,0,3) eq 'TO:'){$line=substr($line,3);} $line=~s,^[\s\t]*,,g; $0=$PCFCONF{'TitleRCPT'}.' '.$line; }
	return smtpd_from_toto($sock, $REFSMTPD, 'TO:', \@{ $REFSMTPD->{TO} }, @_);
}

sub smtpd_cmd_reset {
	my $sock = shift(@_);
	my $REFSMTPD = shift(@_);
	# @_ = ARGS call function
	
	$REFSMTPD->{STATE}='RESET';
	$REFSMTPD->{FROM} = undef;
	@{ $REFSMTPD->{TO} } = ();
	$REFSMTPD->{DATA} = undef;
	
	smtpd_put($sock, "250 Ok");
	return 1; # added: return 0: exit from loop, return 1: continue in loop, return -1 exit from loop with -1
}



1; # return status for include mode


