#!/usr/bin/perl -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults

use File::Basename; # for basename(),
use POSIX qw(strftime); # for strftime(),
use MIME::QuotedPrint; # for decode_qp(),


##########################################################################
# sub perl_content_filter($FHANDLE, \$retcntr, \$retmsgs, \%CPSMTPD, \%CPSOCKINFO, \%CPCONFIG, \%CPHEADERS)
# global   -----
# @param   handle $FHANDLE
# @param   int    \$retcntr    - reference int
# @param   string \$retmsgs    - reference string
# @param   HASH   \%CPSMTPD    - reference HASH
#          string $RCPSMTPD->{'HELO'}
#          string $RCPSMTPD->{'FROM'}
#          LIST   @{$RCPSMTPD->{'TO'}}
#          string $RCPSMTPD->{'DATA'}
# @param   HASH   \%CPSOCKINFO - reference HASH
#          string $RCPSOCKINFO->{'sockaddr'}
#          string $RCPSOCKINFO->{'sockhost'}
#          string $RCPSOCKINFO->{'sockport'}
#          string $RCPSOCKINFO->{'peeraddr'}
#          string $RCPSOCKINFO->{'peerhost'}
#          string $RCPSOCKINFO->{'peerport'}
# @param   HASH   \%CPCONFIG   - reference HASH
#          string $RCPCONFIG->{value}
# @param   HASH   \%CPHEADERS  - reference HASH
#          string $RCPHEADERS->{'From'}
#          string $RCPHEADERS->{'To'}
#          string $RCPHEADERS->{'Subject'}
#          string $RCPHEADERS->{'...'}
# @return  int    - # plugin $status==0  plugin returned status in references $retcntr+$retmsgs
# @return  int    - # plugin $status==-1 means DROP message
# @return  int    - # plugin $status==1  means ACCEPT message (messsage counter=0 and stop analyze message)
# @return  int    - # plugin $status==2  plugin modified content or headers
sub perl_content_filter {
	my ($FHANDLE, $Rretcntr, $Rretmsgs, $RCPSMTPD, $RCPSOCKINFO, $RCPCONFIG, $RCPHEADERS, $REMLSPARSED)=@_;
	my ($retval, $mailfrom, $hdrsfrom, @MAILTOTO, $hdrstoto);
	my ($hrdssize, $bodydata, $bodytext, $hrefperc);
	my ($key, $matchlist, $contenttype, %WORD_PREG_LIST, $pregkey, $pregval, $pregcntr);
	my ($conttext, $conthtml, $html2text, %ATTACH);
	
	$retval=0;
	${$Rretcntr}=0; ${$Rretmsgs}='';
	$mailfrom=$RCPSMTPD->{'FROM'};
	$hdrsfrom=$RCPHEADERS->{'From'};
	@MAILTOTO=@{$RCPSMTPD->{'TO'}};
	$hdrstoto=$RCPHEADERS->{'To'};
	
	$hrdssize=0;
	$hrdssize=index($RCPSMTPD->{DATA},"\x{0D}\x{0A}\x{0D}\x{0A}"); if ($hrdssize>0) { $hrdssize=$hrdssize+4; }
	if ($hrdssize<0) {$hrdssize=index($RCPSMTPD->{DATA},"\x{0D}\x{0D}"); if ($hrdssize>0) { $hrdssize=$hrdssize+2; } }
	if ($hrdssize<0) {$hrdssize=index($RCPSMTPD->{DATA},"\x{0A}\x{0A}"); if ($hrdssize>0) { $hrdssize=$hrdssize+2; } }
	if ($hrdssize<0) {$hrdssize=0;}
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): hrdssize=$hrdssize\n");
	$bodydata=substr($RCPSMTPD->{DATA}, $hrdssize);
	if (defined($RCPHEADERS->{'Content-Transfer-Encoding'}) && lc($RCPHEADERS->{'Content-Transfer-Encoding'}) eq 'quoted-printable') {
		$bodydata=decode_qp($bodydata);
	}
	if (defined($RCPHEADERS->{'Content-Type:charset'}) && lc($RCPHEADERS->{'Content-Type:charset'}) ne 'utf-8') {
		print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): Content-Type:charset=|".$RCPHEADERS->{'Content-Type:charset'}."|\n");
		&myiconv($bodydata, $RCPHEADERS->{'Content-Type:charset'}, 'utf-8');
	}
	$contenttype='';
	foreach $key (sort(keys(%{$RCPHEADERS}))) {
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): $key=|".$RCPHEADERS->{$key}."|\n");
		if (lc($key) eq 'content-type') { $contenttype=$RCPHEADERS->{$key}; }
	}
	
	#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): bodydata=|$bodydata|\n");
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): length(\$bodydata)=".length($bodydata)."\n");
	
	$conttext='';
	$conthtml='';
	$html2text='';
	
	
	
	if (length($contenttype)>0 && index(lc($contenttype),'text/plain')>=0) {
		$conttext=$bodydata;
	}
	
	
	
	if (length($contenttype)>0 && index(lc($contenttype),'text/html')>=0) {
		$conthtml=$bodydata;
		$html2text=$conthtml;
		$html2text=~s,<[^>]*>,,g;
	}
	
	
	
	if (index(lc($contenttype),'multipart/')>=0 || index(lc($contenttype),'boundary')>=0) {
		$conttext=&message_parsed_bodytext($REMLSPARSED);
		$conthtml=&message_parsed_bodyhtml($REMLSPARSED);
		$html2text=$conthtml;
		$html2text=~s,<[^>]*>,,g;
		#%ATTACH  =&message_parsed_files   ($REMLSPARSED);
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): conttext=|$conttext|\n");
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): conthtml=|$conthtml|\n");
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): html2text=|$html2text|\n");
		#foreach $key (keys(%ATTACH)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): key=|\%ATTACH[$key]|\n"); }
	}
	
	
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): length(\$conttext)=".length($conttext)." length(\$conthtml)=".length($conthtml)." length(\$html2text)=".length($html2text)."\n");
	
	
	
	if (defined(${$RCPCONFIG}{'WORD_PREG_LIST'})) {
		$matchlist='';
		%WORD_PREG_LIST=%{${$RCPCONFIG}{'WORD_PREG_LIST'}};
		foreach $pregkey (sort(keys(%WORD_PREG_LIST))) {
			$pregval=$WORD_PREG_LIST{$pregkey};
			#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): pregkey=|$pregkey| pregval=|$pregval|\n");
			$pregkey=~s,\$,\\\$,g;
			# Subject:
			if (length(${$RCPHEADERS}{'Subject'})>0) {
				$pregcntr = () = ${$RCPHEADERS}{'Subject'}=~m/$pregkey/g;
				#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): pregkey=|$pregkey| pregcntr=|$pregcntr|\n");
				if ($pregcntr>0) {
					${$Rretcntr}=${$Rretcntr}+$pregcntr*$pregval; $matchlist=$matchlist."[$pregcntr]"."$pregkey,";
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): MATCH (subject~=m/$pregkey/g) pregval=|$pregval| pregcntr=|$pregcntr| rowstatus=${$Rretcntr}:${$Rretmsgs}\n");
				}
			}
			# TEXT: text/plain
			if (length($conttext)>0) {
				$pregcntr = () = $conttext=~m/$pregkey/g;
				#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): pregkey=|$pregkey| pregcntr=|$pregcntr|\n");
				if ($pregcntr>0) {
					${$Rretcntr}=${$Rretcntr}+$pregcntr*$pregval; $matchlist=$matchlist."[$pregcntr]"."$pregkey,";
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): MATCH (bodydata~=m/$pregkey/g) pregval=|$pregval| pregcntr=|$pregcntr| rowstatus=${$Rretcntr}:${$Rretmsgs}\n");
				}
			}
			# HTML: text/html
			if (length($html2text)>0) {
				$pregcntr = () = $html2text=~m/$pregkey/g;
				#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): pregkey=|$pregkey| pregcntr=|$pregcntr|\n");
				if ($pregcntr>0) {
					${$Rretcntr}=${$Rretcntr}+$pregval; $matchlist=$matchlist."[$pregcntr]"."$pregkey,";
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): MATCH (bodydata~=m/$pregkey/g) pregval=|$pregval| pregcntr=|$pregcntr| rowstatus=${$Rretcntr}:${$Rretmsgs}\n");
				}
			}
		}
		$matchlist=~s,[ ]+$,,g;
		if (length($matchlist)>0) { ${$Rretmsgs}=${$Rretmsgs}."MATCH:(".$matchlist.")"; }
	}
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): status=".${$Rretcntr}.":".${$Rretmsgs}."\n");
	
	return 0;
}



1; # return status for include mode


