#!/usr/bin/perl -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults



##########################################################################
# sub smtpdfilter_init_config()
# global   -----
# @param   HASH   \%CONFIG - reference HASH
# @return  void
sub smtpdfilter_init_config {
	my ($RCONFIG)=@_;
	# @_ = ARGS call function
	#my ()=@_;
	
	${$RCONFIG}{'serveraddr'}='0.0.0.0';
	${$RCONFIG}{'serverport'}=10025;
	${$RCONFIG}{'clientaddr'}='10.41.0.50';
	${$RCONFIG}{'clientport'}=10026;
	
	${$RCONFIG}{'filterpath'}='/usr/lib/smtpdfilter';
	${$RCONFIG}{'filterpref'}='smtpdfilter';
	${$RCONFIG}{'filtersuff'}='pm.pl';
	${$RCONFIG}{'filterperm'}=0;
	
	${$RCONFIG}{'queuedir'}='/var/spool/smtpdfilter/queue';
	${$RCONFIG}{'debuglog'}='/var/log/smtpdfilter/smtpddebug.log';
	${$RCONFIG}{'smtpdlog'}='/var/log/smtpdfilter/smtpdfilter.log';
	${$RCONFIG}{'smtpdpid'}='/var/run/smtpdfilter.pid';
	
	${$RCONFIG}{'smtpdtries'}=5;
	${$RCONFIG}{'loopsleep'}=30;
	
	${$RCONFIG}{'spamcountpkts'}=-20;
	${$RCONFIG}{'spamsubjectprefix'}='';
	
	# smtpdfilter---CLIENT-IPADDR-cidr---pm.pl
	${$RCONFIG}{'nets_cidr_adds_pkts'}=-10;
	${$RCONFIG}{'nets_cidr_file_list'}='/etc/smtpdfilter/nets-cidr-list.conf';
	%{${$RCONFIG}{'NETS_CIDR_LIST'}}=();
	
	# smtpdfilter---CONT-html---pm.pl
	${$RCONFIG}{'body_cont_href_percent'}=0.25;
	${$RCONFIG}{'body_cont_href_pkts'}=-10;
	${$RCONFIG}{'body_cont_html_percent'}=0.25;
	${$RCONFIG}{'body_cont_html_only_pkts'}=-10;
	
	# smtpdfilter---CONT-preg-pattern---pm.pl
	${$RCONFIG}{'word_preg_adds_pkts'}=0;
	${$RCONFIG}{'word_preg_file_list'}='/etc/smtpdfilter/word-preg-pattern.conf';
	%{${$RCONFIG}{'WORD_PREG_LIST'}}=();
	
	# smtpdfilter---HEADers---pm.pl
	${$RCONFIG}{'header_texthtml_pkts'}=-10;
	
	${$RCONFIG}{'header_required_pkts'}=0;
	${$RCONFIG}{'header_required_keys'}='';
	
	${$RCONFIG}{'header_extra_pkts'}=0;
	${$RCONFIG}{'header_extra_keys'}='';
	
	# smtpdfilter---MAIL-FROM---pm.pl
	${$RCONFIG}{'from_mailfrom1diff2hdrsfrom_pkts'}=-10;
	${$RCONFIG}{'from_multiplemailfrom_pkts'}=-10;
	${$RCONFIG}{'from_addr_adds_pkts'}=0;
	${$RCONFIG}{'from_addr_file_list'}='/etc/smtpdfilter/from-addr-list.conf';
	%{${$RCONFIG}{'FROM_ADDR_LIST'}}=();
	
	# smtpdfilter---RCPT-TO---pm.pl
	${$RCONFIG}{'toto_rcptto1notin2hdrstoto_pkts'}=-5;
}



##########################################################################
# sub smtpdfilter_read_config($conffile, \%CONFIG)
# global   -----
# @param   string $conffile
# @param   string $chroot
# @param   HASH   \%CONFIG - reference HASH
# @return  int 1|-1
sub smtpdfilter_read_config {
	my ($conffile, $chroot, $RCONFIG)=@_;
	# @_ = ARGS call function
	my ($fpconf, %FILECONF, $strline, $pregcont);
	my ($key, $val, $sp1, $sp2);
	
	if (!open($fpconf,$conffile)) {
		return -1;
	}
	
	while ($strline=<$fpconf>) {
		$strline=~s,[\s\t\r\n]*$,,g; # rtrim()
		$strline=~s,^[;#].*$,,g; # remove comments
		if (length($strline)<=0) { next; } # skip empty lines
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.':'.(caller(0))[3]."(): strline=|$strline|\n");
		$sp1=index($strline, '=');
		if ($sp1<=0) { next; }
		$key=$strline;
		$val=$strline;
		$key=substr($key, 0, $sp1);
		$val=substr($val, $sp1+1);
		$key=~s,^[\t\r\n]*|[\t\r\n]*$,,g;
		$val=~s,^[\t\r\n]*|[\t\r\n]*$,,g;
		$val=~s,#[^"]*$,,g;
		
		if (substr($val,0,1) eq '"') {
			$sp2=$sp1=1;
			$sp2=index($val, '"', $sp2+1);
			#print(strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.':'.(caller(0))[3]."(): part=|".substr($val,$sp2,1)."|\n");
			while (substr($val,$sp2-1,1) eq '\\' && $sp2>0) {
				$sp1=$sp2;
				$sp2=index($val, '"', $sp2+1);
			}
			#print(strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.':'.(caller(0))[3]."(): |$sp1|=|$val|\n");
			if ($sp1>0) { $val=substr($val, 1, $sp1-1); }
		}
		#print(strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.':'.(caller(0))[3]."(): |$key|=|$val|\n");
		
		
		
		# validate: if (value:OK) then copy values from $FILECONF to $CONFIG
		if ($key eq 'serveraddr') {
			if (length($val)>0) { ${$RCONFIG}{'serveraddr'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value serveraddr=$val"); }
		}
		if ($key eq 'serverport') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'serverport'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value serverport=$val"); }
		}
		if ($key eq 'clientaddr') {
			if (length($val)>0) { ${$RCONFIG}{'clientaddr'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value clientaddr=$val"); }
		}
		if ($key eq 'clientport') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'clientport'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value clientport=$val"); }
		}
		
		if ($key eq 'filterpath') {
			if (length($val)>0) { ${$RCONFIG}{'filterpath'}=$chroot.$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value filterpath=$val"); }
		}
		if ($key eq 'filterpref') {
			if (length($val)>0) { ${$RCONFIG}{'filterpref'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value filterpref=$val"); }
		}
		
		if ($key eq 'filtersuff') {
			if (length($val)>0) { ${$RCONFIG}{'filtersuff'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value filtersuff=$val"); }
		}
		if ($key eq 'filterperm') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'filterperm'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value filterperm=$val"); }
		}
		
		if ($key eq 'queuedir') {
			if (length($val)>0) { ${$RCONFIG}{'queuedir'}=$chroot.$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value queuedir=$val"); }
		}
		if ($key eq 'debuglog') {
			if (length($val)>0) { ${$RCONFIG}{'debuglog'}=$chroot.$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value debuglog=$val"); }
		}
		if ($key eq 'smtpdlog') {
			if (length($val)>0) { ${$RCONFIG}{'smtpdlog'}=$chroot.$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value smtpdlog=$val"); }
		}
		if ($key eq 'smtpdpid') {
			if (length($val)>0) { ${$RCONFIG}{'smtpdpid'}=$chroot.$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value smtpdpid=$val"); }
		}
		
		if ($key eq 'smtpdtries') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'smtpdtries'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value smtpdtries=$val"); }
		}
		
		if ($key eq 'loopsleep') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'loopsleep'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value loopsleep=$val"); }
		}
		
		if ($key eq 'spamcountpkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'spamcountpkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value spamcountpkts=$val"); }
		}
		
		if ($key eq 'spamsubjectprefix') {
			${$RCONFIG}{'spamsubjectprefix'}=$val;
		}
		
		
		
		############################################
		# smtpdfilter---CLIENT-IPADDR-cidr---pm.pl #
		############################################
		if ($key eq 'nets_cidr_adds_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'nets_cidr_adds_pkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value nets_cidr_adds_pkts=$val"); }
			#print($FHANDLE "\${$RCONFIG}{'word_preg_adds_pkts'}=".${$RCONFIG}{'word_preg_adds_pkts'}."\n");exit();
		}
		if ($key eq 'nets_cidr_file_list') {
			if (length($val)>0) { ${$RCONFIG}{'nets_cidr_file_list'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value nets_cidr_file_list=$val"); }
			if (length($chroot)>0) { ${$RCONFIG}{'nets_cidr_file_list'}=$chroot.${$RCONFIG}{'nets_cidr_file_list'}; }
			if (length(${$RCONFIG}{'nets_cidr_file_list'})>0) {
				if (-e ${$RCONFIG}{'nets_cidr_file_list'} && -r ${$RCONFIG}{'nets_cidr_file_list'}) {
					if (open(FILE,${$RCONFIG}{'nets_cidr_file_list'})) {
						read(FILE, $pregcont, 65536);
						close(FILE);
						%{${$RCONFIG}{'NETS_CIDR_LIST'}}=&filelistcont_parse($pregcont, ${$RCONFIG}{'word_preg_adds_pkts'});
						undef($pregcont);
						#foreach my $key (keys(%{${$RCONFIG}{'NETS_CIDR_LIST'}})) { print($FHANDLE "NETS_CIDR_LIST: $key=".${${$RCONFIG}{'NETS_CIDR_LIST'}}{$key}."\n"); }
					}
				} else {
					&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "file word_preg_file_list=".${$RCONFIG}{'word_preg_file_list'}." does not exists or not readable");
				}
			}
		}
		
		
		
		###################################
		# smtpdfilter---CONT-html---pm.pl #
		###################################
		if ($key eq 'body_cont_href_percent') {
			if ($val+0.001>0) { ${$RCONFIG}{'body_cont_href_percent'}=$val+0.001-0.001; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value body_cont_href_percent=$val"); }
		}
		if ($key eq 'body_cont_href_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'body_cont_href_pkts'} =int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value body_cont_href_pkts=$val"); }
		}
		if ($key eq 'body_cont_html_percent') {
			if ($val+0.001>0) { ${$RCONFIG}{'body_cont_html_percent'}=$val+0.001-0.001; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value body_cont_html_percent=$val"); }
		}
		if ($key eq 'body_cont_html_only_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'body_cont_html_only_pkts'} =int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value body_cont_html_only_pkts=$val"); }
		}
		
		
		
		###########################################
		# smtpdfilter---CONT-preg-pattern---pm.pl #
		###########################################
		if ($key eq 'word_preg_adds_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'word_preg_adds_pkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value word_preg_adds_pkts=$val"); }
			#print($FHANDLE "\${$RCONFIG}{'word_preg_adds_pkts'}=".${$RCONFIG}{'word_preg_adds_pkts'}."\n");exit();
		}
		
		if ($key eq 'word_preg_file_list') {
			if (length($val)>0) { ${$RCONFIG}{'word_preg_file_list'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value word_preg_file_list=$val"); }
			if (length($chroot)>0) { ${$RCONFIG}{'word_preg_file_list'}=$chroot.${$RCONFIG}{'word_preg_file_list'}; }
			if (length(${$RCONFIG}{'word_preg_file_list'})>0) {
				if (-e ${$RCONFIG}{'word_preg_file_list'} && -r ${$RCONFIG}{'word_preg_file_list'}) {
					if (open(FILE,${$RCONFIG}{'word_preg_file_list'})) {
						read(FILE, $pregcont, 65536);
						close(FILE);
						%{${$RCONFIG}{'WORD_PREG_LIST'}}=&filelistcont_parse($pregcont, ${$RCONFIG}{'word_preg_adds_pkts'});
						undef($pregcont);
						#foreach my $key (keys(%{${$RCONFIG}{'WORD_PREG_LIST'}})) { print($FHANDLE "WORD_PREG_LIST: $key=".${${$RCONFIG}{'WORD_PREG_LIST'}}{$key}."\n"); }
					}
				} else {
					&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "file word_preg_file_list=".${$RCONFIG}{'word_preg_file_list'}." does not exists or not readable");
				}
			}
		}
		
		
		
		#################################
		# smtpdfilter---HEADers---pm.pl #
		#################################
		if ($key eq 'header_texthtml_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'header_texthtml_pkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value header_texthtml_pkts=$val"); }
		}
		if ($key eq 'header_required_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'header_required_pkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value header_required_pkts=$val"); }
		}
		if ($key eq 'header_required_keys') {
			if (length($val)>0) { ${$RCONFIG}{'header_required_keys'}=$val; }
			#else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value header_required_keys=$val"); }
		}
		if ($key eq 'header_extra_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'header_extra_pkts'}=int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value header_extra_pkts=$val"); }
		}
		if ($key eq 'header_extra_keys') {
			if (length($val)>0) { ${$RCONFIG}{'header_extra_keys'}=$val; }
			#else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value header_extra_keys=$val"); }
		}
		
		
		
		###################################
		# smtpdfilter---MAIL-FROM---pm.pl #
		###################################
		if ($key eq 'from_mailfrom1diff2hdrsfrom_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'from_mailfrom1diff2hdrsfrom_pkts'} =int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value from_mailfrom1diff2hdrsfrom_pkts=$val"); }
		}
		if ($key eq 'from_multiplemailfrom_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'from_multiplemailfrom_pkts'} =int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value from_multiplemailfrom_pkts=$val"); }
		}
		if ($key eq 'from_addr_file_list') {
			if (length($val)>0) { ${$RCONFIG}{'from_addr_file_list'}=$val; }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "EMPTY:value from_addr_file_list=$val"); }
			if (length($chroot)>0) { ${$RCONFIG}{'from_addr_file_list'}=$chroot.${$RCONFIG}{'from_addr_file_list'}; }
			if (length(${$RCONFIG}{'from_addr_file_list'})>0) {
				if (-e ${$RCONFIG}{'from_addr_file_list'} && -r ${$RCONFIG}{'from_addr_file_list'}) {
					if (open(FILE,${$RCONFIG}{'from_addr_file_list'})) {
						read(FILE, $pregcont, 65536);
						close(FILE);
						%{${$RCONFIG}{'FROM_ADDR_LIST'}}=&filelistcont_parse($pregcont, ${$RCONFIG}{'from_addr_adds_pkts'});
						undef($pregcont);
						#foreach my $key (keys(%{${$RCONFIG}{'FROM_ADDR_LIST'}})) { print($FHANDLE "FROM_ADDR_LIST: $key=".${${$RCONFIG}{'FROM_ADDR_LIST'}}{$key}."\n"); }
					}
				} else {
					&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "file from_addr_file_list=".${$RCONFIG}{'from_addr_file_list'}." does not exists or not readable");
				}
			}
		}
		
		
		
		#################################
		# smtpdfilter---RCPT-TO---pm.pl #
		#################################
		if ($key eq 'toto_rcptto1notin2hdrstoto_pkts') {
			if ($val=~m/^[+-]?[0-9]+$/) { ${$RCONFIG}{'toto_rcptto1notin2hdrstoto_pkts'} =int($val); }
			else { &logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "BAD:value toto_rcptto1notin2hdrstoto_pkts=$val"); }
		}
		
		
		
	}
	
	close($fpconf);
	
	
	
	#printf($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): filterpath=".${$RCONFIG}{'filterpath'}."\n");
	
	
	
	return 1;
}



##########################################################################
# sub filelistcont_parse($pregcont, $wordpkts)
# extract_address()
# global   ---
# @param   string $pregcont
# @param   int    $wordpkts
# @return  HASH   %PREGCONT
sub filelistcont_parse {
	my ($pregcont, $wordpkts)=@_;
	# @_ = ARGS call function
	my ($ikey, $strline, $key, $val);
	my (@PREGLIST, %PREGCONT);
	
	%PREGCONT=();
	@PREGLIST=split(/\n/, $pregcont);
	foreach $ikey (0..$#PREGLIST) {
		$strline=$PREGLIST[$ikey];
		$strline=~s,[\r\n]*$,,g; # rtrim(): do not use: [\s\t]!!! for allow words with spaces: " word "
		$strline=~s,^[;#].*$,,g; # remove comments
		if (length($strline)<=0) { next; } # skip empty lines
		#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): strline=|$strline|\n");
		if (rindex($strline,'=')>0 && rindex($strline,'=')+1<length($strline) && ($strline=~m/=[ ]\-*[0-9]+$/||$strline=~m/=[ ]OK$/||$strline=~m/=[ ]NOK$/)) {
			$key=substr($strline, 0, rindex($strline,'='));
			$val=substr($strline, rindex($strline,'=')+1);
			$key=~s,[\s]*$,,g;
			$val=~s,^[\s]*,,g;
			#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): OK:strline=|$strline| key=|$key| val=|$val| === \$PREGCONT{$key}=$val;\n");
			$PREGCONT{$key}=$val;
		} else {
			#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): strline=|$strline| wordpkts=|$wordpkts| === \$PREGCONT{$strline}=$wordpkts;\n");
			$PREGCONT{$strline}=$wordpkts;
		}
	}
	
	return %PREGCONT;
}



1; # return status for include mode


