#!/usr/bin/perl -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults

use File::Basename; # for basename(),
use POSIX qw(pow); # for pow(),



##########################################################################
# sub parse_headers($hdrscont)
# parse_headers()
# global   ---
# @param   string $hdrscont
# @return  HASH   %HEADERS
sub parse_headers {
	my ($hdrscont)=@_;
	# @_ = ARGS call function
	my (%HEADERS, @LINES, $recvind);
	my ($lkey, $key, $val, $line, $sp1);
	
	# FROM=~s,^[^<]+<|>[^>]*$,,g; FROM=~s,>[^<>]+<, ,g;
	# TOTO=~s,^[^<]+<|>[^>]*$,,g; TOTO=~s,>[^<>]+<, ,g;
	$hdrscont=~s,\x{0D}\x{0A},\x{0A},g;
	$hdrscont=~s,\x{0D},\x{0A},g;
	@LINES=split(/\x{0A}/, $hdrscont);
	$recvind=0; $key=''; $val='';
	foreach $lkey (0..$#LINES) {
		$line=$LINES[$lkey];
		#print($FHANDLE "line=$line\n");
		if (length($line)==0) { last; }
		if (ord(substr($line,0,1)) ne 0x20 && ord(substr($line,0,1)) ne 0x09) {
			if (length($key)>0) {
				$HEADERS{$key}=$val;
				if ($key eq 'Received') {
					$HEADERS{$key.$recvind}=$val;
					$recvind++;
				}
			}
			$sp1=index($line, ':');
			if ($sp1>0) {
				$key=substr($line, 0, $sp1);
				$val=substr($line, $sp1+1); $val=~s,^[ ]+,,g;
			} else {
				next;
			}
			
		} else {
			$val.=' '.substr($line,1);
		}
	}
	if (length($key)>0) {
		$HEADERS{$key}=$val;
		if ($key eq 'Received') {
			$HEADERS{$key.$recvind}=$val;
			$recvind++;
		}
	}
	return %HEADERS;
}



##########################################################################
# sub extract_address($address)
# extract_address()
# global   ---
# @param   string $address
# @return  string $address
sub extract_address {
	my ($address)=@_;
	# @_ = ARGS call function
	my ($sp1, $sp2);
	
	$sp1=index($address, '<');
	if ($sp1>=0) { $address=substr($address, $sp1+1); }
	
	$sp1=index($address, '<');
	$sp2=index($address, '>');
	while ($sp2>=0 && $sp1>$sp2) {
		$address=substr($address, 0, $sp2).' '.substr($address, $sp1+1);
		$sp1=index($address, '<');
		$sp2=index($address, '>');
	}
	
	$sp2=index($address, '>');
	if ($sp2>=0) { $address=substr($address, 0, $sp2); }
	return $address;
}



##########################################################################
# sub check_ipaddr_match_network()
# check_ipaddr_match_network()
# global   ---
# @param   string $ipaddr
# @param   string $network
# @return  int - 1:ipaddr mach network, 0:ipaddr does not match network
sub check_ipaddr_match_network {
	my ($ipaddr, $network)=@_;
	# @_ = ARGS call function
	my ($netaddr, $netmask, $netmhex);
	my ($ipanum, $netmnum, $netnum);
	
	# $inetaddr=inet_aton("10.0.0.3"); 
	# printf("inet=%02X.%02X.%02X.%02X\n", ord(substr($inetaddr,0,1)), ord(substr($inetaddr,1,1)), ord(substr($inetaddr,2,1)), ord(substr($inetaddr,3,1)));
	# 0A.00.00.03 = 10.0.0.3
	# 
	# $inetaddr=inet_aton("10.0.0.3");
	# printf("ipaddr=%d.%d.%d.%d\n", ord(substr($inetaddr,0,1)), ord(substr($inetaddr,1,1)), ord(substr($inetaddr,2,1)), ord(substr($inetaddr,3,1)));
	# 
	# 10.0.0.3
	# $inetaddr=inet_aton("10.0.0.3");
	# printf("ipaddr=%08X\n", ord(substr($inetaddr,0,1))<<24 | ord(substr($inetaddr,1,1))<<16 | ord(substr($inetaddr,2,1))<<8 | ord(substr($inetaddr,3,1)));
	# ipaddr=0A000003
	if (index($network,'/')>0) {
		# ipaddr==X.X.X.X/NET
		$netaddr=substr($network, 0, index($network,'/'));
		$netmask=substr($network, index($network,'/')+1);
		$netmask=int($netmask);
		if ($netmask==0) { $netmask=32; }
	} else {
		# ipaddr===X.X.X.X   ===   ipaddr===X.X.X.X/32
		$netaddr=$network;
		$netmask=32;
	}
	#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): network=|$network| netaddr=|$netaddr| netmask=|$netmask|\n");
	
	$netmnum=(pow(2,32)-1)-(pow(2,32- $netmask )-1);
	$netmhex=sprintf("0x%08X", $netmnum);
	#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): netmask=|$netmask| netmhex=|$netmhex|\n");
	$ipanum=unpack(N=>pack(CCCC=>split(/\./=>$ipaddr)));
	$netnum=unpack(N=>pack(CCCC=>split(/\./=>$netaddr)));
	#print(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): ipanum=|$ipanum| netnum=|$netnum|\n");
	
	#printf(basename(__FILE__).":".__LINE__.":PID[$$]:".(caller(0))[3]."(): ipanum=|$ipanum|0x%08X| netnum=|$netnum|0x%08X|\n", $ipanum, $netnum);
	if (($ipanum & $netmnum) == ($netnum & $netmnum)) {
		return 1;
	} else {
		return 0;
	}
}



####################
##### utils.pl #####
####################
##########################################################################
# sub printr(@LIST)
# printr() like PHP print_r()
# global:  ---
# @param   reference SCALAR|ARRAY|HASH
# @return  void
sub printr {
	my ($argref)=@_;
	# @_ = ARGS call function
	my ($key);
	
	if      (ref($argref) eq 'SCALAR') {
		if (defined(${$argref})) { print("SCALAR=".${$argref}."\n"); }
		else                     { print("SCALAR=UNDEFINED!!!\n"); }
	} elsif (ref($argref) eq 'ARRAY') {
		print("LIST(\n");
		foreach $key (0..$#{$argref}) {
			if (defined(${$argref}[$key])) { print("$key => ".${$argref}[$key]."\n"); }
			else                           { print("$key => UNDEFINED!!!\n"); }
		}
		print(")\n");
	} elsif (ref($argref) eq 'HASH') {
		print("HASH(\n");
		foreach $key (sort(keys(%{$argref}))) {
			if (defined(${$argref}{$key})) { print("$key => ".${$argref}{$key}."\n"); }
			else                           { print("$key => UNDEFINED!!!\n"); }
		}
		print(")\n");
	} else {
		print("UNKNOWN arg ref type=".ref($argref)."\n");
	}
}



##########################################################################
# sub createBoundary()
# global   ---
# @param   int    $bndrsize
# @return  string
sub createBoundary {
	my ($bndrsize)=@_;
	# @_ = ARGS call function
	my ($sec, $usec);
	my ($boundary);
	
	($sec, $usec)=gettimeofday();
	$boundary=md5_hex(strftime("%D %T",localtime(time())) . ($sec+$usec/1e6));
	if (defined($bndrsize) && $bndrsize<32) { $boundary=substr($boundary, 0, $bndrsize); }
	return $boundary;
}





#####################
##### fileio.pl #####
#####################
##########################################################################
# sub logsinfo_open($filelogs)
# global:  LOGSINFO (file handle)
# @param   string $filelogs
# @return  int    status
sub logsinfo_open {
	my ($filelogs)=@_;
	# @_ = ARGS call function
	#my ();
	
	if (!defined($filelogs)||length($filelogs)<=0) { $filelogs="/dev/stdout"; }
	if (!open(LOGSINFO, ">>$filelogs")) { return -1; } # printf(STDERR "Could not open file: $filelogs, Reason $!\n");
	#LOGSINFO->autoflush(1); # use flock(FILE, LOCK_EX) rather than autoflush(1) ???
	return 0;
}



##########################################################################
# sub logsinfo_write($callargs, $message)
# global:  LOGSINFO (file handle)
# @param   string $callargs (   __FILE__:__LINE__:(caller(0))[3]   ||   basename(__FILE__):__LINE__:(caller(0))[3]   )
# @param   string $message
# @return  string $strline
sub logsinfo_write {
	my ($callargs, $message)=@_;
	# @_ = ARGS call function
	my ($datetime, $strline, $callfile, $callline, $callsubr);
	
	$datetime=strftime('%Y/%m/%d %H:%M:%S', localtime());
	$message=~s,[\t\r\n]*,,g;
	# 2010/09/25: when caled as __FILE__:__LINE__:(caller(0))[3]
	# 2010/09/25: 2008/03/03 10:00:00 /path/to/file.pl:LINE:callback() 10.0.0.5 example.com "http://example.com/index.php"
	if (index($callargs,':')>0) {
		$callfile=substr($callargs, 0, index($callargs,':'));
		# 2010/09/25: skip logging path to file running as ./file.pl
		if (substr($callfile,0,2) eq "./") { $callfile=substr($callfile, 2); }
		$callargs=substr($callargs, index($callargs,':')+1);
		if (index($callargs,':')>0) {
				$callline=substr($callargs, 0, index($callargs,':'));
				$callsubr=substr($callargs, index($callargs,':')+1);
				if (substr($callsubr,0,6) eq "main::") { $callsubr=substr($callsubr, 6); }
		} else { $callline="0"; $callsubr="undefined";  }
	} else { $callfile="undefined"; $callline="0"; $callsubr=$callargs; }
	# 2008/03/03 10:00:00 callback() 10.0.0.5 example.com "http://example.com/index.php"
	$strline="$datetime $callfile:$callline:$callsubr(): $message";
	#print(STDERR "[$$] $strline\n");
	# NOTICE(2008/09/08): what happend if here file will be locked by other process
	# flock semantics: it waits indefinitely until the lock is granted, and that its locks merely advisory
	# programs that do not also use flock may modify files locked with flock
	no strict 'subs';
	flock(LOGSINFO, LOCK_EX); ##### try to lock file
	print(LOGSINFO $strline."\n");
	flock(LOGSINFO, LOCK_UN);
	use strict 'subs';
	
	return $strline;
}



##########################################################################
# sub logsinfo_close()
# global:  LOGSINFO (file handle)
# @param   void
# @return  int    status
sub logsinfo_close {
	#my ();
	# @_ = ARGS call function
	#my ();
	
	if (!close(LOGSINFO)) { return -1; } # printf(STDERR "Could not close file, Reason $!\n");
	return 0;
}



1; # return status for include mode


