#!/usr/bin/perl -w

use strict;      # force definations for all vars an subroutines
use warnings;    # force check of unset variables in expressions
use diagnostics; # give var/code line number on faults

use File::Basename; # for basename(), dirname(),
use Cwd qw(cwd abs_path); # for cwd(), abs_path(),
use POSIX qw(strftime); # for strftime(),
use Time::HiRes qw(gettimeofday usleep ualarm tv_interval nanosleep clock_gettime clock_getres clock); # for gettimeofday(), usleep(), ualarm, tv_interval, nanosleep, clock_gettime, clock_getres clock, # Time::HiRes::clock_nanosleep(): unimplemented in this platform, "stat" is not exported by the Time::HiRes module
use Digest::MD5 qw(md5 md5_hex md5_base64); # for md5(), md5_hex(), md5_base64(),
use Fcntl qw(F_GETFL F_SETFL LOCK_SH LOCK_EX LOCK_UN O_NONBLOCK); # import LOCK_* constants, LOCK_SH, LOCK_EX, LOCK_UN, O_NONBLOCK
use Sys::Hostname; # for hostname(),
use POSIX qw(:signal_h :errno_h :sys_wait_h); # for WNOHANG,
use POSIX qw(setuid setgid getuid getgid); # for setuid(), setgid(), getuid(), getgid(),
use Socket; # for AF_UNSPEC, AF_INET, AF_INET6, AF_UNIX, SOCK_STREAM, SOCK_DGRAM, sockaddr_in(), sockaddr_un(), inet_ntoa(), inet_aton(), getpeername(), getsockname(), bind(), listen(), accept(),
use IO::Socket; # for IO::Socket::INET->new(),
use Net::SMTP; # for Net::SMTP->new($smtp_addr, Port=>$smtp_port, Timeout=>$smtp_timeout, Debug=>0),

# ----------------------------------------------------------
#                      instalation
# ----------------------------------------------------------

# after-queue content filtering
# -----------------------------
# /etc/postfix/master.cf
# ----------------------
# smtp      inet  n       -       n       -       -       smtpd
# --------------------------------------------------------------
# smtp      inet  n       -       n       -       -       smtpd -o content_filter=content-filter:10.0.0.3:10025 -o receive_override_options=no_address_mappings
#
# /etc/postfix/master.cf
# ----------------------
# # Send out to content-filter
# content-filter         unix  -       -        n     -       2       smtp
#
# # postfix/master[1937]: fatal: bind 0.0.0.0 port 10025: Address already in use
# # Receive from content-filter
# 127.0.0.1:10026 inet  n       -       n       -       -       smtpd -o smtpd_enforce_tls=no -o content_filter= -o smtpd_client_restrictions= -o smtpd_helo_restrictions= -o smtpd_sender_restrictions= -o smtpd_recipient_restrictions=permit_mynetworks,reject -o mynetworks=127.0.0.0/8



# before-queue content filtering
# reject email af the end of DATA
# -------------------------------
#
# /etc/postfix/master.cf
# ----------------------
# smtp      inet  n       -       n       -       -       smtpd
# --------------------------------------------------------------
# smtp      inet  n       -       n       -       -       smtpd -o smtpd_proxy_filter=10.0.0.3:10025 -o receive_override_options=no_address_mappings -o smtpd_client_connection_count_limit=10
#
# # postfix/master[1937]: fatal: bind 0.0.0.0 port 10025: Address already in use
# # Receive from content-filter
# 127.0.0.1:10026 inet  n       -       n       -       -       smtpd -o smtpd_enforce_tls=no -o content_filter= -o smtpd_client_restrictions= -o smtpd_helo_restrictions= -o smtpd_sender_restrictions= -o smtpd_recipient_restrictions=permit_mynetworks,reject -o mynetworks=127.0.0.0/8



my $self=$0;
my $rootpath=dirname(abs_path($0));
my $toolpath=dirname($rootpath).'/lib/smtpdfilter';
my $conffile=dirname(dirname($rootpath)).'/etc/smtpdfilter/smtpdfilter.conf';
my $basename=basename($0);
require("$toolpath/daemonlib.pl");
require("$toolpath/smtpdutils.pl");
require("$toolpath/smtpdconf.pl");
require("$toolpath/message-parse.pl");
require("$toolpath/perl-Net-SMTP-Server-Client.pl");



my (%PID2FH, @PID2LS, $serverquesize);
my ($FHANDLE, $workmode, %CONFIG);

my ($chroot, $socktimeout);
my ($smtpd, $listtmout);



$chroot='';
$workmode=0;
$socktimeout=30;
$serverquesize=5;
%CONFIG=();
&daemon_init($0, @ARGV);
if (defined($ARGV[0]) && $ARGV[0] eq '-f') {   if (defined($ARGV[1])) { shift(@ARGV); $conffile=shift(@ARGV); } else { &smtpdfilter_usage(); }   }
if (defined($ARGV[0]) && $ARGV[0] eq '-X') { $workmode=1; shift(@ARGV); }
if (defined($ARGV[0]) && $ARGV[0] eq '--chroot') { if (defined($ARGV[1])) { shift(@ARGV); $chroot=shift(@ARGV); } else { &smtpdfilter_usage(); } }



#($CONFIG{'serveraddr'}, $CONFIG{'serverport'}, $CONFIG{'clientaddr'}, $CONFIG{'clientport'})=('0.0.0.0', 10025, '10.41.0.54', 10026);
&smtpdfilter_init_config(\%CONFIG);
if (&logsinfo_open('/dev/stdout')<0) { print(STDERR "Unable open log file: /dev/stdout.\n"); exit(1); }
if (&smtpdfilter_read_config($conffile,$chroot,\%CONFIG)!=1) { print(STDERR "Unable read config file: $conffile.\n"); exit(1); }
&logsinfo_close();

if (!defined($CONFIG{'serveraddr'})   ||!defined($CONFIG{'serverport'})   ||!defined($CONFIG{'clientaddr'})   ||!defined($CONFIG{'clientport'})   ) { print(STDERR "Bad config file: \$CONFIG{'serveraddr'}||\$CONFIG{'serverport'}||\$CONFIG{'clientaddr'}||\$CONFIG{'clientport'} not defined.\n"); exit(1); }
if (  length($CONFIG{'serveraddr'})<=0||  length($CONFIG{'serverport'})<=0||  length($CONFIG{'clientaddr'})<=0||  length($CONFIG{'clientport'})<=0) { print(STDERR "Bad config file: \$CONFIG{'serveraddr'}||\$CONFIG{'serverport'}||\$CONFIG{'clientaddr'}||\$CONFIG{'clientport'} not defined.\n"); exit(1); }



&daemon_setps();
if (&logsinfo_open($CONFIG{'smtpdlog'})<0) { print(STDERR "Unable open log file: $CONFIG{'smtpdlog'}.\n"); exit(1); }
$SIG{'HUP'}='smtpdfilter_handler';

# O_NONBLOCK + Timeout=>1 - how long wait for connections, set short time for ofter call waitpid() and remove zombie from processlist
$listtmout=0.25; # 0.25 means 4 x in one second waitpid will be called
$smtpd=IO::Socket::INET->new(LocalAddr=>$CONFIG{'serveraddr'}, LocalPort=>$CONFIG{'serverport'}, Type=>SOCK_STREAM, Proto=>'tcp', Listen=>$serverquesize, Timeout=>$listtmout, ReuseAddr=>1);
if (!defined($smtpd)) { print(STDERR "Unable bind to $CONFIG{'serveraddr'}:$CONFIG{'serverport'}, Reason: $@.\n"); exit(2); }
binmode($smtpd);
$smtpd->sockopt(SOL_SOCKET, O_NONBLOCK);

if ($workmode==1) {
	if (!open($FHANDLE,">>/dev/stdout")) { print(STDERR "Unable open /dev/stdout\n"); exit(1); }
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.'__main__', "&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);");
	&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.'__main__', "&smtpdfilter_server_run(\$smtpd, \%CONFIG, $serverquesize, $socktimeout, $listtmout);");
	&smtpdfilter_server_run($smtpd, \%CONFIG, $serverquesize, $socktimeout, $listtmout);
	close($FHANDLE);
} else {
	if (!open($FHANDLE,">>".$CONFIG{'debuglog'})) { print(STDERR "Unable open $CONFIG{'debuglog'}\n"); exit(1); }
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.'__main__', "&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);");
	&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.'__main__', "&daemon_run(\&smtpdfilter_server_run, $CONFIG{'smtpdpid'}, \$smtpd, \%CONFIG, $serverquesize, $socktimeout, $listtmout);");
	&daemon_run(\&smtpdfilter_server_run, $CONFIG{'smtpdpid'}, $smtpd, \%CONFIG, $serverquesize, $socktimeout, $listtmout);
	close($FHANDLE);
}
&logsinfo_close();





sub smtpdfilter_usage {
	print("Usage: $0 [OPTIONS]\n");
	print("       OPTIONS:\n");
	print("       -f <configfile> path to config file\n");
	print("       -X foreground mode\n");
	print("       --chroot <path--to-chroot-directory> path to config file\n");
	exit(1);
}



#######################
#####   handler   #####
#######################
##########################################################################
# sub smtpdfilter_handler()
# global   $chroot, $smtpd, %CONFIG
# @param   int    $sig
# @return  void
sub smtpdfilter_handler {
	my ($sig)=@_; # 1st argument is signal name
	# @_ = ARGS call function
	my ($newsock, $oldsock);
	
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "Caught a SIG$sig, Reloading configuration.");
	
	if (&smtpdfilter_read_config($conffile,$chroot,\%CONFIG)==1) {
		
		if (length($chroot)>0) { $CONFIG{'filterpath'}=$chroot.$CONFIG{'filterpath'}; }
		if (length($chroot)>0) { $CONFIG{'queuedir'}  =$chroot.$CONFIG{'queuedir'}; }
		if (length($chroot)>0) { $CONFIG{'debuglog'}  =$chroot.$CONFIG{'debuglog'}; }
		if (length($chroot)>0) { $CONFIG{'smtpdlog'}  =$chroot.$CONFIG{'smtpdlog'}; }
		if (length($chroot)>0) { $CONFIG{'smtpdpid'}  =$chroot.$CONFIG{'smtpdpid'}; }
		if (length($chroot)>0) { $CONFIG{'nets_cidr_file_list'}=$chroot.$CONFIG{'nets_cidr_file_list'}; }
		if (length($chroot)>0) { $CONFIG{'word_preg_file_list'}=$chroot.$CONFIG{'word_preg_file_list'}; }
		if (length($chroot)>0) { $CONFIG{'from_addr_file_list'}=$chroot.$CONFIG{'from_addr_file_list'}; }
		
		if ((-e $CONFIG{'smtpdlog'} && -w $CONFIG{'smtpdlog'}) || -w dirname($CONFIG{'smtpdlog'})) {
			&logsinfo_close();
			#print($FHANDLE "smtpdfilter_handler(): open($CONFIG{'smtpdlog'}) OK\n");
			&logsinfo_open($CONFIG{'smtpdlog'});
		} else {
			print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.": if ((-e $CONFIG{'smtpdlog'} && -w $CONFIG{'smtpdlog'}) || -w dirname($CONFIG{'smtpdlog'})):else: smtpdlog=$CONFIG{'smtpdlog'} NOT WRITABLE\n");
		}
		
		#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): smtpd->sockhost():smtpd->sockport()=".$smtpd->sockhost().':'.$smtpd->sockport()."\n");
		if ($smtpd->sockhost() ne $CONFIG{'serveraddr'} && $smtpd->sockport()!=$CONFIG{'serverport'}) {
			# 2013/02/04: at first we open new sock, and then close old sock,
			# so we can not open sock on the same port but other address, because port is in use !!!
			# so we can only open sock, when port has changed also
			print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): IO::Socket::INET->new(LocalAddr=>$CONFIG{'serveraddr'}, LocalPort=>$CONFIG{'serverport'}, Type=>SOCK_STREAM, Proto=>'tcp', Listen=>$serverquesize, Timeout=>$listtmout, ReuseAddr=>1)\n");
			$newsock=IO::Socket::INET->new(LocalAddr=>$CONFIG{'serveraddr'}, LocalPort=>$CONFIG{'serverport'}, Type=>SOCK_STREAM, Proto=>'tcp', Listen=>$serverquesize, Timeout=>$listtmout, ReuseAddr=>1);
			if (!defined($newsock)) {
				#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Unable bind to $CONFIG{'serveraddr'}:$CONFIG{'serverport'}, Reason: $@.\n");
				&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "Unable bind to $CONFIG{'serveraddr'}:$CONFIG{'serverport'}, Reason: $@.");
			} else {
				#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Bind to $CONFIG{'serveraddr'}:$CONFIG{'serverport'} OK\n");
				$oldsock=$smtpd;
				$smtpd=$newsock;
				binmode($smtpd);
				$smtpd->sockopt(SOL_SOCKET, O_NONBLOCK);
				$oldsock->close();
				undef($oldsock);
			}
		}
		
		&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);");
		&smtpdfilter_server_spoolrun(\%CONFIG, $socktimeout);
		
	} else {
		&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "Failed smtpdfilter_read_config($conffile), Reason: $!, Keep old configuration!");
	}
}



###########################
#####   spool:queue   #####
###########################
##########################################################################
# sub smtpdfilter_server_spoolread()
# global   $FHANDLE, %PID2FH, @PID2LS
# @param   void
# @return  void
sub smtpdfilter_server_spoolread {
	my ($queuefile)=@_;
	# @_ = ARGS call function
	my ($FHRD, $line, $REFSMTPD);
	
	%{$REFSMTPD}=();
	
	if (!open($FHRD,$queuefile)) { return undef; }
	
	while ($line=<$FHRD>) {
		$line=~s,[\r\n]*$,,g;
		if (substr($line,0,length('CLIENT '   )) eq 'CLIENT '   ) { $REFSMTPD->{'CLIENT'}=substr($line,length('CLIENT '   )); }
		if (substr($line,0,length('HELO '     )) eq 'HELO '     ) { $REFSMTPD->{'HELO'}  =substr($line,length('HELO '     )); }
		if (substr($line,0,length('MAILFROM: ')) eq 'MAILFROM: ') { $REFSMTPD->{'FROM'}  =substr($line,length('MAILFROM: ')); }
		if (substr($line,0,length('RCPTTO: '  )) eq 'RCPTTO: '  ) { @{$REFSMTPD->{'TO'}} =split(/ /,substr($line,length('RCPTTO: '  ))); }
		if (substr($line,0,length('DATA:'     )) eq 'DATA:'     ) { last; }
	}
	
	read($FHRD, $REFSMTPD->{'DATA'}, (stat($queuefile))[7]);
	
	close($FHRD);
	
	return $REFSMTPD;
}



##########################################################################
# sub smtpdfilter_server_spoolrun()
# global   $FHANDLE, %PID2FH, @PID2LS
# @param   void
# @return  void
sub smtpdfilter_server_spoolrun {
	my ($RCONFIG, $conntimeout)=@_;
	# @_ = ARGS call function
	my ($DIR, $entry, $queuefile);
	my ($pid, $RSOCKINFO, $REFSMTPD, $i, $status, $queueid);
	
	$RSOCKINFO->{'peerhost'}='0.0.0.0';
	$RSOCKINFO->{'peerport'}=0;
	
	if (opendir($DIR,$CONFIG{'queuedir'})) {
		while ($entry=readdir($DIR)) {
			
			
			
			if ($entry eq '.' || $entry eq '..') { next; }
			$queuefile=$CONFIG{'queuedir'}.'/'.$entry;
			print($FHANDLE "$self: \$entry=readdir(\$DIR): \$entry=|$entry| \$queuefile=|$queuefile|\n");
			$REFSMTPD=&smtpdfilter_server_spoolread($queuefile);
			if (!defined($REFSMTPD)) { next; }
			print($FHANDLE "CLIENT=".$REFSMTPD->{CLIENT}."\n");
			print($FHANDLE "HELO=".$REFSMTPD->{HELO}."\n");
			print($FHANDLE "FROM=".$REFSMTPD->{FROM}."\n");
			print($FHANDLE "TOTO=".join('|',@{$REFSMTPD->{TO}})."\n");
			
			
			
			$pid=fork();
			# die "fork() failed: $!" unless defined $pid;
			# die "fork() failed: $!" if not defined $pid;
			# if (!defined($pid)) { die("fork() failed: $!"); }
			
			
			if (defined($pid)) {
				# fork success
				if ($pid) {
					
					$PID2FH{$pid}="smtpdfilter_server_run()";
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): conninfo=".$PID2FH{$pid}."\n");
					
				} else {
					
					
					
					$queueid=createBoundary(10);
					$queueid=~tr/[a-z]/[A-Z]/;
					
					$RSOCKINFO->{'peerhost'}='0.0.0.0';
					$RSOCKINFO->{'peerport'}=0;
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): sending back ".$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}." ...\n");
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): SET PS=|".$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] Connecting '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}.' ...'."|\n");
					$0=$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] Connecting '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}.' ...';
					for ($i=0;$i<$RCONFIG->{'smtpdtries'};$i++) {
						$status=&smtpdfilter_server_send($RCONFIG->{'clientaddr'}, $RCONFIG->{'clientport'}, $socktimeout, $queueid, $REFSMTPD->{HELO}, $REFSMTPD->{FROM}, \@{$REFSMTPD->{TO}}, $REFSMTPD->{DATA});
						print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): LOOP: &smtpdfilter_server_send():status=$status\n");
						if ($status==1) { last; } else { sleep($RCONFIG->{'loopsleep'}); }
					}
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): SET PS=|".$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] QUIT '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}."|\n");
					$0=$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] QUIT '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'};
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): END &smtpdfilter_server_send():status=$status\n");
					
					print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): unlink(|$queuefile|)\n");
					unlink($queuefile);
					
					exit(0);
					
				};
			} else {
					# fork failed, try in main instance (this causes block accepting messages until below code is running)
					&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "fork() failed, Reason: $@.");
					#exit(1); # 2014/05/23: smtpdfilter.pl: HUP ===> smtpdfilter_handler() ===> smtpdfilter_server_spoolrun() ===> exit(): Causes exit smtpdfilter,
			}
			
			
			
			
		} # while ($entry=readdir($DIR))
		closedir($DIR);
	} else { # if (opendir($DIR,$CONFIG{'queuedir'}))
		&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "Unable opendir queuedir=$CONFIG{'queuedir'}, Reason: $@.");
	} # if (opendir($DIR,$CONFIG{'queuedir'})): else:
}



######################
#####   server   #####
######################
##########################################################################
# sub smtpdfilter_server_run()
# global   $FHANDLE, %PID2FH, @PID2LS
# @param   string $listenaddr
# @param   int    $listenport
# @return  void
sub smtpdfilter_server_run {
	my ($smtpd, $RCONFIG, $listenquesize, $conntimeout, $listtmout)=@_;
	# @_ = ARGS call function
	my ($pid, $innconn, %SOCKINFO);
	
	
	
	# 2014/05/23 run as nobody
	# daemon_run create pid file, we need to be root to can to that
	# so we change to nobody after create pid file, just here
	my $uid=(getpwnam('nobody'))[2];
	my $gid=(getgrnam('nobody'))[2];
	# if (getuid()!=0) (0=root): causes in: setgid()|setuid() = -1 EPERM (strace perl)
	# NOTE: first setuid(), second setgid() does not work as needed: getuid():uid=99 getgid():gid=0
	# NOTE: so we need: first setgid(), second setuid(),
	if (defined($uid)) { setgid($gid); }
	if (defined($gid)) { setuid($uid); }
	
	
	
	# IO::Socket::INET->new() moved up for detect posibility to bind before daemonize and display error
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): wait for connections on ".$RCONFIG->{'serveraddr'}.':'.$RCONFIG->{'serverport'}." (send back to: ".$RCONFIG->{'clientaddr'}.":".$RCONFIG->{'clientport'}.") (quesize=$listenquesize)\n");
	while (($innconn=$smtpd->accept()) || 1) {
		@PID2LS=keys(%PID2FH);
		foreach $pid (@PID2LS) {
			$pid=waitpid($pid, WNOHANG);
			if ($pid>0 && defined($PID2FH{$pid})) { delete($PID2FH{$pid}); }
		}
		if (!defined($innconn)) { usleep(1000000*$listtmout); next; }
		print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): while (client=server->accept())\n");
		$SOCKINFO{'sockaddr'}=$innconn->sockaddr();
		$SOCKINFO{'sockhost'}=$innconn->sockhost();
		$SOCKINFO{'sockport'}=$innconn->sockport();
		$SOCKINFO{'peeraddr'}=$innconn->peeraddr();
		$SOCKINFO{'peerhost'}=$innconn->peerhost();
		$SOCKINFO{'peerport'}=$innconn->peerport();
		
		if (!defined($innconn) || !defined($SOCKINFO{'sockaddr'}) || !defined($SOCKINFO{'sockport'}) || !defined($SOCKINFO{'sockhost'}) || !defined($SOCKINFO{'peeraddr'}) || !defined($SOCKINFO{'peerport'}) || !defined($SOCKINFO{'peerhost'})) {
			print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Broken Connection:\n");
			# if (defined($SOCKINFO{'sockaddr'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): sa=$SOCKINFO{'sockaddr'}\n"); }
			if (defined($SOCKINFO{'sockhost'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): sh=$SOCKINFO{'sockhost'}\n"); }
			if (defined($SOCKINFO{'sockport'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): sp=$SOCKINFO{'sockport'}\n"); }
			# if (defined($SOCKINFO{'peeraddr'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): pa=$SOCKINFO{'peeraddr'}\n"); }
			if (defined($SOCKINFO{'peerhost'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): ph=$SOCKINFO{'peerhost'}\n"); }
			if (defined($SOCKINFO{'peerport'})) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): pp=$SOCKINFO{'peerport'}\n"); }
			next;
		}
		print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Connection: ACCEPT: $SOCKINFO{'peerhost'}:$SOCKINFO{'peerport'} => $SOCKINFO{'sockhost'}:$SOCKINFO{'sockport'}\n");
		
		
		
		$pid=fork();
		# die "fork() failed: $!" unless defined $pid;
		# die "fork() failed: $!" if not defined $pid;
		# if (!defined($pid)) { die("fork() failed: $!"); }
		if (defined($pid)) {
			# fork success
			if ($pid) {
				
				undef($innconn); # it must be to close properly connection to client, without connection end with broken
				$PID2FH{$pid}="$SOCKINFO{'peerhost'}:$SOCKINFO{'peerport'} => $SOCKINFO{'sockhost'}:$SOCKINFO{'sockport'}";
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): conninfo=".$PID2FH{$pid}."\n");
				
			} else {
				
				binmode($innconn);
				&smtpdfilter_server_child($innconn, \%SOCKINFO, $RCONFIG);
				exit(0);
				
			};
		} else {
				# fork failed, try in main instance (this causes block accepting messages until below code is running)
				binmode($innconn);
				&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "fork() failed, Reason: $@.");
				&smtpdfilter_server_child($innconn, \%SOCKINFO, $RCONFIG);
		}
	} # while (($innconn=$smtpd->accept()) || 1)
	
	return 0;
}



##########################################################################
# sub smtpdfilter_server_child()
# global   $FHANDLE, %CONFIG
# @param   IO::Socket::Net $innconn (object)
# @param   HASH \%SOCKINFO - reference to HASH
# @return  void
sub smtpdfilter_server_child {
	my ($innconn, $RSOCKINFO,  $RCONFIG)=@_;
	# @_ = ARGS call function
	my (          %CPSOCKINFO, %CPCONFIG, $REFSMTPD, %CPSMTPD);
	my ($pid, $status, $queuefile, $queueid, $i, $infoline, $headline, $sp1, $sp2);
	
	$queueid=createBoundary(10);
	$queueid=~tr/[a-z]/[A-Z]/;
	
	
	
	
	
	# RECV
	#$REFSMTPD=smtpd_new($innconn, Banner=>hostname(), Timeout=>10, TitleARGS=>1, TitleCONN=>$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] CONNECTING', TitleHELO=>$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] HELO', TitleMAIL=>$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] MAIL FROM', TitleRCPT=>$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] RCPT TO', TitleDATA=>$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] DATA');
	$REFSMTPD=smtpd_new($innconn, Banner=>hostname(), Timeout=>10, TitleARGS=>1, TitleCONN=>sprintf('%s [%s:%s] CONNECTING', $basename, $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}), TitleHELO=>sprintf('%s [%s:%s] HELO',   $basename, $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}), TitleMAIL=>sprintf('%s [%s:%s] MAIL FROM',   $basename, $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}), TitleRCPT=>sprintf('%s [%s:%s] RCPT TO',   $basename, $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}), TitleDATA=>sprintf('%s [%s:%s] DATA',   $basename, $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}));
	$status=&smtpd_sockrecv($innconn, $REFSMTPD, $queueid);
	if ($REFSMTPD->{STATE} eq 'END-OF-DATA') {
		# parse DATA and return response
		#if (defined($REFSMTPD->{DATA}) && index($REFSMTPD->{DATA},'before-queue')>=0) {
		#	&smtpd_put($innconn, "451 4.3.0 <".${$REFSMTPD->{TO}}[0].">: Temporary lookup failure (in reply to END of DATA command)");
		#	$status=-1;
		#} else {
		#	&smtpd_put($innconn, "250 Ok: queued as ".$queueid);
		#}
		&smtpd_put($innconn, "250 Ok: queued as ".$queueid);
	}
	$innconn->close();
	if ($status<=0) { return -1; }
	
	$infoline=sprintf('Connect: %s:%s helo=%s from=%s to=%s', $RSOCKINFO->{'peerhost'}, $RSOCKINFO->{'peerport'}, $REFSMTPD->{HELO}, $REFSMTPD->{FROM}, join(',',@{$REFSMTPD->{TO}}));
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "$queueid: $infoline");
	
	# WRITE QUEUEFILE
	$queuefile=$CONFIG{'queuedir'}.'/'.$queueid;
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): queuefile=$queuefile\n");
	if (-e $CONFIG{'queuedir'} && -w $CONFIG{'queuedir'}) {
		if (open(QUEUEFILE,">$queuefile")) {
			print(QUEUEFILE "CLIENT ".$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}."\n");
			print(QUEUEFILE "HELO ".$REFSMTPD->{HELO}."\n");
			print(QUEUEFILE "MAILFROM: ".$REFSMTPD->{FROM}."\n");
			print(QUEUEFILE "RCPTTO: ".join('|',@{$REFSMTPD->{TO}})."\n");
			print(QUEUEFILE "DATA:\n");
			print(QUEUEFILE $REFSMTPD->{DATA});
			close(QUEUEFILE);
		}
	}
	
	$0=$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] processing';
	
	# DEBUG
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): HELO=".$REFSMTPD->{HELO}."\n");
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): FROM=".$REFSMTPD->{FROM}."\n");
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): TO  =".join(' ',@{$REFSMTPD->{TO}})."\n");
	#print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): DATA=".$REFSMTPD->{DATA}."\n");
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): length(DATA)=".length($REFSMTPD->{DATA})."\n");
	
	
	
	
	
	# FILTER-Plugins
	my ($revaddr, %HEADERS, %CPHEADERS, %EMLSPARSED, $hdrsline);
	my (@PLUGINS, $pluginfile, $pluginname, $smtpdPCFcntr, $smtpdPCFmsgs, $plugcntr, $plugmsgs);
	my ($filterpath, $filterpref, $filtersuff)=($RCONFIG->{'filterpath'}, $RCONFIG->{'filterpref'}, $RCONFIG->{'filtersuff'});
	if (defined($RCONFIG->{'filterpath'})) { @PLUGINS=<$filterpath/$filterpref*$filtersuff>; }
	else { @PLUGINS=(); }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '."\n");
	$smtpdPCFcntr=0; $smtpdPCFmsgs='';
	
	#%HEADERS=&parse_headers( substr($REFSMTPD->{DATA},0,8192) );
	%HEADERS=&message_content_headers( substr($REFSMTPD->{DATA},0,16384) );
	%EMLSPARSED=&message_content_parse( $REFSMTPD->{DATA} );
	
	
	
	
	
	foreach $pluginfile (@PLUGINS) {
		if (-e $pluginfile && -r $pluginfile) {
			print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): pluginfile=$pluginfile\n");
			###open(STDERR, ">/dev/null");
			if (! eval { require($pluginfile); } ) {
				&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "$queueid: pluginfile=".basename($pluginfile)." FAILED: eval { require($pluginfile); }");
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): FAILED: eval { require($pluginfile); }\n");
				next;
			}
			if (defined(&perl_content_filter)) {
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): pluginfile=$pluginfile perl_content_filter(): DEFINED\n");
				$plugcntr=0; $plugmsgs='';
				# passing $REFSMTPD, $RSOCKINFO, \%HEADERS as reference causes plugin can modify this values
				# to avoid that create copy this values
				%CPSMTPD=();
				$CPSMTPD{'HELO'}=$REFSMTPD->{'HELO'};
				$CPSMTPD{'FROM'}=$REFSMTPD->{'FROM'};
				@{$CPSMTPD{'TO'}}=@{$REFSMTPD->{'TO'}};
				$CPSMTPD{'DATA'}=$REFSMTPD->{'DATA'};
				%CPSOCKINFO=%{$RSOCKINFO};
				%CPHEADERS=%HEADERS;
				%CPCONFIG=%{$RCONFIG};
				# pass as reference $plugcntr, $plugmsgs, we want use values to pass in headers result returned by plugin
				$status=&perl_content_filter($FHANDLE, \$plugcntr, \$plugmsgs, \%CPSMTPD, \%CPSOCKINFO, \%CPCONFIG, \%CPHEADERS, \%EMLSPARSED);
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): perl_content_filter():status=$status $plugcntr:$plugmsgs\n");
				&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "$queueid: pluginfile=".basename($pluginfile)." status=$status $plugcntr:$plugmsgs");
				
				# plugin $status==-1 means DROP message
				if ($status==-1) { undef(%CPSMTPD); undef(%CPSOCKINFO); undef(%HEADERS); print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): perl_content_filter():status=$status DROPPED\n"); return; }
				
				# plugin $status==1  means ACCEPT message (messsage counter=0 and stop analyze message)
				if ($status==1) { $smtpdPCFcntr=$plugcntr; $smtpdPCFmsgs=$plugmsgs; last; }
				
				# plugin $status==2  plugin modified content or headers
				if ($CONFIG{'filterperm'}==1 && $status==2) { $REFSMTPD->{HELO}=$CPSMTPD{HELO}; $REFSMTPD->{FROM}=$CPSMTPD{FROM}; @{$REFSMTPD->{TO}}=@{$CPSMTPD{TO}}; $REFSMTPD->{DATA}=$CPSMTPD{DATA}; %HEADERS=%CPHEADERS; }
				
				# plugin $status==0  plugin returned $retcntr+$retmsgs in references
				$smtpdPCFcntr=$smtpdPCFcntr+$plugcntr;
				#if    (length($plugmsgs)>45) { $smtpdPCFmsgs=$smtpdPCFmsgs.substr($plugmsgs,0,42).'...'.' '; }
				#elsif (length($plugmsgs)> 0) { $smtpdPCFmsgs=$smtpdPCFmsgs.$plugmsgs.' '; }
				if (length($plugmsgs)> 0) {
					while (ord(substr($plugmsgs,0,1))==0x20 || ord(substr($plugmsgs,0,1))==0x09) { $plugmsgs=substr($plugmsgs, 1); }
					$pluginname=basename($pluginfile);
					if (substr($pluginname,0,length($filterpref)) eq $filterpref) { $pluginname=substr($pluginname, length($filterpref)); }
					if (substr($pluginname,0 - length($filtersuff)) eq $filtersuff) { $pluginname=substr($pluginname, 0, 0 - length($filtersuff)); }
					while (substr($pluginname,0,1) eq '-' || substr($pluginname,0,1) eq '_') { $pluginname=substr($pluginname, 1); }
					while (substr($pluginname,-1,1) eq '-' || substr($pluginname,-1,1) eq '_') { $pluginname=substr($pluginname, 0, -1); }
					#if (length($smtpdPCFmsgs)>0) { $smtpdPCFmsgs=$smtpdPCFmsgs."\n"; } # if not empty append new line at the end (to avoid append new line at the beggining)
					$smtpdPCFmsgs=$smtpdPCFmsgs."\t$pluginname: $smtpdPCFcntr=".$plugmsgs."\n";
				}
				
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): summarystatus=$smtpdPCFcntr:$smtpdPCFmsgs\n");
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '."\n");
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '."\n");
				print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '."\n");
				
				undef(&perl_content_filter);
			} # if (defined(&perl_content_filter))
		} # if (-e $pluginfile && -r $pluginfile)
	} # foreach $pluginfile (@PLUGINS)
	
	
	
	
	
	# append header info to message content
	# if ($smtpdPCFcntr<=$RCONFIG->{'spamcountpkts'} && length($smtpdPCFmsgs))
	# NOTICE: $smtpdPCFmsgs = "\tFILTER: pkts:msgs\n" . "\tFILTER: pkts:msgs\n" . "\tFILTER: pkts:msgs\n"
	if ($smtpdPCFcntr<=$RCONFIG->{'spamcountpkts'}) {
		while (ord(substr($smtpdPCFmsgs,-1,1))==0x20 || ord(substr($smtpdPCFmsgs,-1,1))==0x09) { $plugmsgs=substr($plugmsgs, 0, -1); } # removes spaces from the end of message
		$hdrsline="Received-PCF: $smtpdPCFcntr SPAM (smtpdfilter: Perl Content Filter)\n";
		if (length($smtpdPCFmsgs)>10) { $hdrsline.=$smtpdPCFmsgs; }
	} else {
		$hdrsline="Received-PCF: $smtpdPCFcntr NONE (smtpdfilter: Perl Content Filter)\n";
		if (length($smtpdPCFmsgs)>10) { $hdrsline.=$smtpdPCFmsgs; }
	}
	$REFSMTPD->{DATA}=$hdrsline.$REFSMTPD->{DATA};
	$hdrsline=~s,[\s\r\n]*$,,g; # for display below
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): hdrsline=|$hdrsline|\n");
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "$queueid: PCF: $hdrsline");
	if (length($RCONFIG->{'spamsubjectprefix'})>0) {
		if (defined($HEADERS{'Subject'})) {
			$sp1=index($REFSMTPD->{DATA}, "\x0ASubject");
			if ($sp1>0) { $sp2=index($REFSMTPD->{DATA}, "\x0A", $sp1+1); } else { $sp2=-1; }
			if ($sp1>0 && $sp2>0) {
				$headline=$RCONFIG->{'spamsubjectprefix'};
				$headline=~s,%d,$smtpdPCFcntr,g;
				if (substr($REFSMTPD->{DATA},$sp2-1) eq "\x0D") { $sp2=$sp2-1; }
				$REFSMTPD->{DATA}=substr($REFSMTPD->{DATA},0,$sp1) ."X-Subject-Orig: $HEADERS{'Subject'}\nSubject: $headline$HEADERS{'Subject'}". substr($REFSMTPD->{DATA},$sp2);
			}
		} else {
			
		}
	}
	
	
	
	
	
	# SEND
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): sending back ".$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}." ...\n");
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): SET PS=|".$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] Connecting '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}.' ...'."|\n");
	$0=$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] Connecting '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}.' ...';
	for ($i=0;$i<$RCONFIG->{'smtpdtries'};$i++) {
		$status=&smtpdfilter_server_send($RCONFIG->{'clientaddr'}, $RCONFIG->{'clientport'}, $socktimeout, $queueid, $REFSMTPD->{HELO}, $REFSMTPD->{FROM}, \@{$REFSMTPD->{TO}}, $REFSMTPD->{DATA});
		print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): LOOP: &smtpdfilter_server_send():status=$status\n");
		if ($status==1) { unlink($queuefile); last; } else { sleep($RCONFIG->{'loopsleep'}); }
	}
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): SET PS=|".$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] QUIT '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'}."|\n");
	$0=$basename.' ['.$RSOCKINFO->{'peerhost'}.':'.$RSOCKINFO->{'peerport'}.'] QUIT '.$RCONFIG->{'clientaddr'}.':'.$RCONFIG->{'clientport'};
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): END &smtpdfilter_server_send():status=$status\n");
	&logsinfo_write(basename(__FILE__).':'.__LINE__.':'.(caller(0))[3], "$queueid: &smtpdfilter_server_send():status=$status");
}



# /etc/postfix/main.cf:smtpd_enforce_tls = no
# -------------------------------------------
# smtpdfilter.pl:341:PID[667]:main::smtpdfilter_server_send(): $smtpc=Net::SMTP->new(10.41.0.54, Port=>10026, Timeout=>30, Debug=>0);
# smtpdfilter.pl:345:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::hello(xnd.nat.wbcd.pl):
# smtpdfilter.pl:347:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::hello(xnd.nat.wbcd.pl): result=|cen05kde.xen.wbcd.pl|
# smtpdfilter.pl:350:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::mail(<user@xnd.nat.wbcd.pl>):
# smtpdfilter.pl:352:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::mail(<user@xnd.nat.wbcd.pl>): result=|1|
# smtpdfilter.pl:355:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::recipient(<sp@cen05kde.xen.wbcd.pl>):
# smtpdfilter.pl:357:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::recipient(<sp@cen05kde.xen.wbcd.pl>): result=|1|
# smtpdfilter.pl:360:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::data():
# smtpdfilter.pl:362:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::data(): result=|1|
# smtpdfilter.pl:365:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::datasend($datacont):
# smtpdfilter.pl:367:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::datasend($datacont): result=|1|
# smtpdfilter.pl:370:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::dataend():
# smtpdfilter.pl:372:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::dataend(): result=|1|
# smtpdfilter.pl:375:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::quit():
# smtpdfilter.pl:377:PID[667]:main::smtpdfilter_server_send(): Net::SMTP::quit(): result=|1|
# smtpdfilter.pl:313:PID[667]:main::smtpdfilter_server_child(): END &smtpdfilter_server_send():status=1
# 
# /etc/postfix/main.cf:smtpd_enforce_tls = yes
# --------------------------------------------
# smtpdfilter.pl:341:PID[780]:main::smtpdfilter_server_send(): $smtpc=Net::SMTP->new(10.41.0.54, Port=>10026, Timeout=>30, Debug=>0);
# smtpdfilter.pl:345:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::hello(xnd.nat.wbcd.pl):
# smtpdfilter.pl:347:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::hello(xnd.nat.wbcd.pl): result=|cen05kde.xen.wbcd.pl|
# smtpdfilter.pl:350:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::mail(<user@xnd.nat.wbcd.pl>):
# smtpdfilter.pl:352:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::mail(<user@xnd.nat.wbcd.pl>): result=||
# smtpdfilter.pl:355:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::recipient(<sp@cen05kde.xen.wbcd.pl>):
# smtpdfilter.pl:357:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::recipient(<sp@cen05kde.xen.wbcd.pl>): result=|0|
# smtpdfilter.pl:360:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::data():
# smtpdfilter.pl:362:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::data(): result=||
# smtpdfilter.pl:365:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::datasend($datacont):
# smtpdfilter.pl:367:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::datasend($datacont): result=|1|
# smtpdfilter.pl:370:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::dataend():
# smtpdfilter.pl:372:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::dataend(): result=|1|
# smtpdfilter.pl:375:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::quit():
# smtpdfilter.pl:377:PID[780]:main::smtpdfilter_server_send(): Net::SMTP::quit(): result=|1|
# smtpdfilter.pl:313:PID[780]:main::smtpdfilter_server_child(): END &smtpdfilter_server_send():status=1

##########################################################################
# sub smtpdfilter_server_send()
# global   -----
# @param   string $connaddr
# @param   string $connport
# @param   string $conntimeout
# @param   HASH  \%SOCKINFO - reference HASH
# @param   string $queueid
# @param   string $hdrshelo
# @param   string $hdrsfrom
# @param   LIST   \@REFTOTO
# @param   LIST   \@REFCONT - reference LIST
# @param   string $datacont
# @return  int
sub smtpdfilter_server_send {
	my ($connaddr, $connport, $conntimeout, $queueid, $hdrshelo, $hdrsfrom, $REFTOTO, $datacont)=@_;
	# @_ = ARGS call function
	my (@HDRSTOTO, $smtpc, $hostname, $result);
	
	$hostname=$ENV{'HOSTNAME'};
	if (!defined($hostname)||length($hostname)<=0) { $hostname=hostname(); }
	@HDRSTOTO=@{$REFTOTO};
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): \$smtpc=Net::SMTP->new($connaddr, Port=>$connport, Timeout=>$conntimeout, Debug=>0);\n");
	$smtpc=Net::SMTP->new($connaddr, Port=>$connport, Timeout=>$conntimeout, Debug=>0);
	if (!defined($smtpc)) { return -1; }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::hello($hostname):\n");
	$result=$smtpc->hello($hostname);
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::hello($hostname): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::hello($hostname): NOT DEFINED result\n"); }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::mail($hdrsfrom):\n");
	$result=$smtpc->mail($hdrsfrom);
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::mail($hdrsfrom): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::mail($hdrsfrom): NOT DEFINED result\n"); }
	if (defined($result)) { if (length($result)==0||$result ne "1") { return -2; } }
	else                  { return -2; }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::recipient(@HDRSTOTO):\n");
	$result=$smtpc->recipient(@HDRSTOTO);
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::recipient(@HDRSTOTO): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::recipient(@HDRSTOTO): NOT DEFINED result\n"); }
	if (defined($result)) { if (length($result)==0||$result ne "1") { return -3; } }
	else                  { return -3; }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::data():\n");
	$result=$smtpc->data();
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::data(): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::data(): NOT DEFINED result\n"); }
	if (defined($result)) { if (length($result)==0||$result ne "1") { return -4; } }
	else                  { return -4; }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::datasend(\$datacont):\n");
	$result=$smtpc->datasend($datacont);
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::datasend(\$datacont): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::datasend(\$datacont): NOT DEFINED result\n"); }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::dataend():\n");
	$result=$smtpc->dataend();
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::dataend(): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::dataend(): NOT DEFINED result\n"); }
	
	print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::quit():\n");
	$smtpc->quit();
	if (defined($result)) { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::quit(): result=|$result|\n"); }
	else                  { print($FHANDLE strftime('%Y/%m/%d %H:%M:%S',localtime()).' '.basename(__FILE__).':'.__LINE__.":PID[$$]:".(caller(0))[3]."(): Net::SMTP::quit(): NOT DEFINED result\n"); }
	
	return 1;
}


